using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.ComponentModel.DataAnnotations.Schema;
using IRaCIS.Core.Domain.Share;
using Newtonsoft.Json;

namespace IRaCIS.Core.Domain.Models
{
    [Table("Trial")]
    public partial class Trial : Entity, IAuditUpdate, IAuditAdd, ISoftDelete
    {
        public Trial()
        {
            ClinicalTrialProjectDetails = new HashSet<TrialStatusDetail>();
            TrialDicList = new List<TrialDictionary>();
        }

        [JsonIgnore]
        public List<TaskConsistentRule> TaskConsistentRuleList { get; set; }
        [JsonIgnore]
        public List<SubjectUser> SubjectDoctorUserList { get; set; } = new List<SubjectUser>();
        [JsonIgnore]
        public List<VisitTask> VisitTaskList { get; set; } = new List<VisitTask>() { };
        [JsonIgnore]
        public List<TrialSiteSurvey> TrialSiteSurveyList { get; set; } = new List<TrialSiteSurvey>();

        [JsonIgnore]
        public List<TrialDocument> TrialDocumentList { get; set; }
        [JsonIgnore]
        public List<Enroll> EnrollList { get; set; } = new List<Enroll>();
        [JsonIgnore]
        public List<Workload> WorkloadList { get; set; } = new List<Workload>();
        [JsonIgnore]
        public List<TrialUser> TrialUserList { get; set; } = new List<TrialUser>();
        [JsonIgnore]
        public List<ReadingQuestionCriterionTrial> ReadingQuestionCriterionTrialList { get; set; } = new List<ReadingQuestionCriterionTrial>();
        [JsonIgnore]
        public List<Subject> SubjectList { get; set; } = new List<Subject>();
        [JsonIgnore]
        public List<DicomStudy> StudyList { get; set; } = new List<DicomStudy>();
        [JsonIgnore]
        public List<TrialSite> TrialSiteList { get; set; } = new List<TrialSite>();
        [JsonIgnore]
        public List<TrialSiteUser> TrialSiteUserList { get; set; } = new List<TrialSiteUser>();

        [JsonIgnore]
        [ForeignKey("DeclarationTypeId")]
        public Dictionary DeclarationType { get; set; }

        public Guid DeclarationTypeId { get; set; } = Guid.Empty;

        public Guid IndicationTypeId { get; set; } = Guid.Empty;
        public Guid? PhaseId { get; set; } = Guid.Empty;

        [JsonIgnore]
        [ForeignKey("IndicationTypeId")]
        public Dictionary IndicationType { get; set; }

        [JsonIgnore]
        [ForeignKey("PhaseId")]
        public Dictionary Phase { get; set; }

        [JsonIgnore]
        [ForeignKey("SponsorId")]
        public Sponsor Sponsor { get; set; }
        [JsonIgnore]
        [ForeignKey("CROId")]
        public CRO CRO { get; set; }
        [JsonIgnore]
        [ForeignKey("ReviewModeId")]
        public Dictionary ReviewMode { get; set; }

        [JsonIgnore]

        public List<ClinicalDataTrialSet> clinicalDataTrialSets { get; set; } = new List<ClinicalDataTrialSet> { };
        [JsonIgnore]
        public virtual ICollection<TrialStatusDetail> ClinicalTrialProjectDetails { get; set; }
        [JsonIgnore]
        public virtual ICollection<TrialDictionary> TrialDicList { get; set; }



        [StringLength(100)]
        public string TrialCode { get; set; } = string.Empty;

        public int Code { get; set; }

        [StringLength(512)]
        public string Indication { get; set; } = string.Empty;

        //һ״̬
        //[ConcurrencyCheck]
        public int TrialEnrollStatus { get; set; }


        //״̬
        [StringLength(500)]
        public string TrialStatusStr { get; set; } = StaticData.TrialState.TrialInitializing;

        [JsonIgnore]
        public List<TrialStateChange> TrialStateChangeList { get; set; } = new List<TrialStateChange>();



        public Guid? CROId { get; set; } = Guid.Empty;

        public Guid? SponsorId { get; set; } = Guid.Empty;
        public Guid? ReviewModeId { get; set; } = Guid.Empty;


        [StringLength(500)]
        public string ProjectCycle { get; set; } = string.Empty;


        public int ExpectedPatients { get; set; }

        public decimal TimePointsPerPatient { get; set; }

        public int GRRReviewers { get; set; }

        public int TotalReviewers { get; set; }

        [StringLength(500)]
        public string ReviewProtocol { get; set; } = string.Empty;

        [StringLength(500)]
        public string MessageFromClient { get; set; } = string.Empty;

        public string Note { get; set; } = string.Empty;


        public string ReviewProtocolName { get; set; } = string.Empty;
        public string MessageFromClientName { get; set; } = string.Empty;
        public int Expedited { get; set; }

        public DateTime CreateTime { get; set; }

        public Guid CreateUserId { get; set; }

        public DateTime UpdateTime { get; set; }

        public Guid UpdateUserId { get; set; }
        public AttendedReviewerType AttendedReviewerType { get; set; } = AttendedReviewerType.CN;//0ȫйҽ 1ҽ  2йҽҲҽ

        public bool VisitPlanConfirmed { get; set; }




        /// <summary>
        /// ߱ž
        /// </summary>
        public string SubjectCodeRule { get; set; } = "5λɣǰ2λΪıţ3λΪ˳ţEDC¼ıűһ";
        /// <summary>
        ///  Ƿ ߱Ź  
        /// </summary>
        public bool IsNoticeSubjectCodeRule { get; set; } = true;

        /// <summary>
        /// Ƿ л׼ʱ䣨״θҩʱ䣩
        /// </summary>
        public bool IsHaveFirstGiveMedicineDate { get; set; } = true;

        /// <summary>
        /// Ƿ  
        /// </summary>
        public bool IsHaveSubjectAge { get; set; } = false;


        /// <summary>
        /// Ƿ ȷ
        /// </summary>
        public bool IsEnrollementQualificationConfirm { get; set; } = false;


        /// <summary>
        /// ƻ
        /// </summary>
        public string OutEnrollmentVisitName { get; set; } = "EOT";



        /// <summary>
        /// Ƿ ֤Ƭ  
        /// </summary>
        public bool IsVerifyVisitImageDate { get; set; } = true;


        /// <summary>
        /// ٴϢ 1ϵͳ¼  2ϵͳ¼+PDF  0
        /// </summary>
        public int ClinicalInformationTransmissionEnum { get; set; } = 1;

        /// <summary>
        /// Ƿ ٴϢ
        /// </summary>
        public bool IsCRAAuditClinicalInformation { get; set; } = false;

        /// <summary>
        /// QC  0 1 2˫  
        /// </summary>
        public TrialQCProcess QCProcessEnum { get; set; } = TrialQCProcess.DoubleAudit;

        /// <summary>
        /// ӰһԺ˲
        /// </summary>
        public bool IsImageConsistencyVerification { get; set; } = true;

        /// <summary>
        /// Ӱ񵼳
        /// </summary>
        public bool IsImageExport { get; set; } = false;


        public bool IsSubjectSecondCodeView { get; set; }


        ///// <summary>
        ///// 1 Mint2 PACS  
        ///// </summary>

        //public int ImagePlatform { get; set; } = 1;

        //Ƭʽ
        public int ReadingMode { get; set; } = 1;

        ////Ƭ 
        //public ReadingMethod ReadingType { get; set; } = ReadingMethod.Double;


        //public bool IsGlobalReading { get; set; } = true;

        ///// <summary>
        ///// ٲƬ
        ///// </summary>
        //public bool? IsArbitrationReading { get; set; } = true;

        //public bool IsClinicalReading { get; set; }


        ///// <summary>
        ///// ٲù
        ///// </summary>
        //public ArbitrationRule ArbitrationRule { get; set; } = ArbitrationRule.None;


        public int ChangeDefalutDays { get; set; } = 5;


        /// <summary>
        /// Ŀ
        /// </summary>
        public bool IsImageReplicationAcrossTrial { get; set; } = false;




        public string BodyPartTypes { get; set; } = "ʲ|Բ||ز|/¸|ǻ|ȫ|";



        public string Modalitys { get; set; } = "CT|MR|BoneScan|Photograph|PET|X-ray|US";


        public string PreliminaryAuditReuploadText { get; set; } = string.Empty;

        public string ReviewAuditReuploadText { get; set; } = string.Empty;



        //PD չǷʾ   ÷ Ƿʾ PDչ (Ӷ״̬)
        public bool IsPDProgressView { get; set; }

        //о
        public string ResearchProgramNo { get; set; }

        //ʵ
        public string ExperimentName { get; set; }

        //еλ
        public string MainResearchUnit { get; set; }

        // PI
        public string HeadPI { get; set; }

        public bool IsUrgent { get; set; }

        /// <summary>
        /// Ŀ 1 ʽĿ0 ʽĿ  2ѵĿ
        /// </summary>time
        public TrialType TrialType { get; set; }

        //public string TempCode { get; set; }


        public int PlanSiteCount { get; set; }

        public int PlanVisitCount { get; set; }



        public DateTime? TrialFinishedTime { get; set; }

        public bool IsSubjectSexView { get; set; } = false;

        //Suject  Editҳ Ƿʾ Ӽ 
        public bool IsSubjectExpeditedView { get; set; } = false;


        public bool IsTrialStart { get; set; } = false;
        public bool IsDeleted { get; set; }


        //QC 

        public User QCQuestionConfirmedUser { get; set; }
        public Guid? QCQuestionConfirmedUserId { get; set; }
        public DateTime? QCQuestionConfirmedTime { get; set; }

        /// <summary>
        /// Ŀʱ
        /// </summary>
        public DateTime? TrialFinishTime { get; set; }


        public int? DigitPlaces { get; set; } = 2;


        public bool IsTrialProcessConfirmed { get; set; }
        public bool IsTrialBasicLogicConfirmed { get; set; }
        public bool IsTrialUrgentConfirmed { get; set; }

        public bool IsQCQuestionConfirmed { get; set; }


        public DateTime? DeletedTime { get; set; }
        public Guid? DeleteUserId { get; set; }

        ////
        //public TaskAllocateObj TaskAllocateObjEnum { get; set; }

        ////Զ
        //public bool IsFollowVisitAutoAssign { get; set; } = true;

        ////ȫԶ
        //public bool IsFollowGlobalVisitAutoAssign { get; set; } = true;

        //public bool IsFollowJudgeTaskAutoAssign { get; set; } = true;

        //public TaskAllocateDefaultState FollowJudgeTaskAutoAssignDefaultState { get; set; } = TaskAllocateDefaultState.Allocated;

        ////ԶĬ״̬
        //public TaskAllocateDefaultState FollowVisitAutoAssignDefaultState { get; set; } = TaskAllocateDefaultState.Allocated;

        ////ȫԶĬ״̬
        //public TaskAllocateDefaultState FollowGlobalVisitAutoAssignDefaultState { get; set; } = TaskAllocateDefaultState.Allocated;

        ////Ƭʾ

        //public ReadingTaskViewMethod ReadingTaskViewEnum { get; set; }

        ////ƬʾǷ˳
        //public bool IsReadingTaskViewInOrder { get; set; } = true;

        ///// <summary>
        ///// ƬǷʾϢ
        ///// </summary>
        //public bool IsReadingShowSubjectInfo { get; set; } = false;

        ///// <summary>
        ///// ƬǷʾ
        ///// </summary>
        //public bool IsReadingShowPreviousResults { get; set; } = false;


        /// <summary>
        /// ͬٴʱ
        /// </summary>
        public DateTime? SyncClinicalDataTime { get; set; }

        /// <summary>
        /// ȷҽѧ
        /// </summary>

        public bool IsConfirmMedicineQuestion { get; set; } = false;

        /// <summary>
        /// ͼǷбע
        /// </summary>
        public bool IsImageIabeled { get; set; }


        ///// <summary>
        ///// ƬϢǩʱ
        ///// </summary>
        //public DateTime? ReadingInfoSignTime { get; set; }

        ///// <summary>
        ///// Ƭ
        ///// </summary>
        //public ReadingTool? ReadingTool { get; set; }

        //public Guid? ReviewTypeId { get; set; } = Guid.Empty;

        //[ForeignKey("ReviewTypeId")]
        //public Dictionary ReviewType { get; set; }


        //public Guid? QCSecondConfirmedUserId { get; set; }
        //public DateTime? QCSecondConfirmedTime { get; set; }

        //public int QCQuestionConfirmState { get; set; }
    }
}
